function [defaultString, gameDescription, functionsDescription, minGmActions, maxGmActions] = game_default( varargin )

% Name: game_default.m
% Location: MALT/GUI/game_default.m
% Purpose:  Stores and returns the default input string for 
%           creating games in Gamut
% Input Parameters:	Name of the game generator
% Output Parameters: Default format for running the game
%                    Description of the game

gameName = varargin{1};
funcNeeded = 0; % Flag for whether function descriptions are needed or not

switch gameName
case 'ArmsRace'
	defaultString = '-actions B -c_func C -c_params [D] -b_func E -b_params [F] -low_act G';
	gameDescription = {'Create an instance of an Arms Race game. Payoffs in this game are symmetric and calculated by using the formula -C(x) + B(x - y) where x is the level of arms the player in question has chosen, y is the level of arms his opponent has chosen, and C and B are user-specified functions.' 'Please note that in order for the game to meet the definition of an Arms Race common in economics literature it must be the case that B is smooth and concave and C is at least smooth. Choose your functions accordingly.' '' 'Game Parameters:' '-actions: symmetric version (see section 3.2)' '-c_func: the name of the function class to use for the cost function C. The function supplied should be SMOOTH in order to stick to the strict definition of an Arms Race.' '-c_params: parameters to be handed off to the cost function C, must be enclosed in [].' '-b_func: the name of the function class to use for the B function. The function supplied should be SMOOTH AND CONCAVE in order to stick to the strict definition of an Arms Race.' '-b_params: parameters to be handed off to the B function, must be enclosed in [].' '-low_act: lower bound on the players action range. Must be > 0 and < 1000.'};
	funcNeeded = 1;
	minGmActions = 2;
	maxGmActions = inf;

case 'BertrandOligopoly'
	defaultString = '-players A -actions B -cost_func C -cost_params D -demand_func E -demand_params F';
	gameDescription = {'Creates an instance of a Bertrand Oligopoly using arbitrary cost and demand functions.' 'In the Bertrand Oligopoly, each player offering the object at the lowest price p will receive a payoff of p*(D(p)/m) - C(D(p)/m) where D is the demand function, C is the cost function, and m is the number of players who offered the object at this price.' 'Please note that the demand function should be non-negative and decreasing.' '' 'Game Parameters:' '-players' '-actions: symmetric version (see section 3.2)' '-cost_func: the name of the function class to use for the cost function.' '-cost_params: parameters to be handed off to the cost function, must be enclosed in [].' '-demand_func: the name of the function class to use for the demand function.' '-demand_params: parameters to be handed off to the demand function, must be enclosed in [].'};
	funcNeeded = 1;
	minGmActions = 2;
	maxGmActions = inf;

case 'Chicken'
	defaultString = '';
	gameDescription = {'Creates a 2x2 Chicken Game' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters: No parameters.'};
	minGmActions = 2;
	maxGmActions = 2;

case 'CollaborationGame'
	defaultString = '-players A';
	gameDescription = {'Creates a collaboration game.' 'By our definition, both coordination and collaboration games are common payoff yet not always symmetric. The highest payoffs are for all outcomes in which every player chooses the same action. In the collaboration game (unlike in the general coordiation game) these outcomes will all yield the same payoff.' 'By default, payoffs will be in the range [-100, 100] with all coordinated payoffs set to 100 for each player. To change this range, use the normalization or integer payoff options.' '' 'Game Parameters: - players'};
	minGmActions = 2;
	maxGmActions = 2;

case 'CoordinationGame'
	defaultString = '-players A';
	gameDescription = {'Creates a Coordination Game.' 'By our definition, coordination games are common payoff yet not always symmetric. The highest payoffs are for all outcomes in which every player chooses the same action, although it is not always the case that all of these outcomes yield the same payoffs. (See collaboration games.)' 'By default, payoffs will be in the range [-100, 100] with coordinated payoffs positive and uncoordinated payoffs negative. To change this range, use the normalization or integer payoff options.' '' 'Game Parameters: - players'};
	minGmActions = 2;
	maxGmActions = 2;

case 'CournotDuopoly'
	defaultString = '-actions B -cost_func1 C -cost_params1 [D] -cost_func2 E -cost_params2 [F] -p_func G -p_params [H]';
	gameDescription = {'Create an instance of the Cournot Duopoly using arbitrary cost and inverse demand functions.' 'In order for the problem to make sense, the cost functions used should be increasing. If C1 and C2 are cost functions and P is the inverse demand function then if player 1 plays y1 and player 2 plays y2, the payoff to player 1 will be P(y1+y2)y1-C1(y1) and the payoff to player 2 will be P(y1 + y2)y2 - C2(y2)' 'Although this formulation could be extended to more than two players, this is generally not done in practice so we limit the players to 2.' '' 'Game Parameters:' '- actions: symmetric version (see section 3.2)' '- cost func1: the name of the function class to use for the cost function for the first player.' '-cost params1: parameters to be handed off to the cost function for player 1, must be enclosed in [].' '-cost func2: the name of the function class to use for the cost function for the second 	player.' '-cost params2: parameters to be handed off to the cost function for player 2, must be enclosed in [].' '-p_func: the name of the function class to use for the inverse demand function P.' '-p_params: parameters to be handed off to the inverse demand function, must be enclosed in [].'};
	funcNeeded = 1;
	minGmActions = 2;
	maxGmActions = inf;

case 'CovariantGame'
	defaultString = '-players A -actions B -r C';
	gameDescription = {'Creates a game with the given number of players with payoffs distributed normally (0,1) with covariance r' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-players' '-actions: non-symmetric version (see section 3.2)' '-r: covariance of any two players payoffs in the same action profile. Must be betweeen -1(players-1) and 1.'};
	minGmActions = 2;
	maxGmActions = inf;

case 'DispersionGame'
	defaultString = '-players A -actions B';
	gameDescription = {'Returns a strong dispersion game which is both action and player symmetric as well as common payoff. An entropy calculation is used in order to determine when one outcome is more dispersed than another, although this could easily be replaced by standard deviation or a similar test.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-players' '-actions: symmetric version (see section 3.2)'};
	minGmActions = 2;
	maxGmActions = inf;

case 'GuessTwoThirdsAve'
	defaultString = '-players A -actions B';
	gameDescription = {'Creates an instance of the game in which all players guess a number trying to come as close as possible to two thirds of the average of the numbers guessed by all players.' 'By default, the payoffs for this game are in the range from 0 to 100.0, where the player whose guess comes closest to two thirds of the average receives 100.0 and the others receive 0. If more there is a tie, the payoff amount is split. To change the range of payoffs you can use the normalization or integer payoff options.' '' 'Game Parameters:' '-players' '-actions: symmetric version (see section 3.2)'};
	minGmActions = 2;
	maxGmActions = inf;

case 'GrabTheDollar'
	defaultString = '-actions B';
	gameDescription = {'Creates an instance of game Grab the Dollar. In this game, there is a prize (or "dollar") that both players are free to grab at any time, where actions represent the chosen times. If both players grab for it at the same time, they will rip the price and both will receive the low payoff. If one chooses a time earlier than the other (i.e. chooses a strictly lower action number number) then he will receive the price (and thus the high payoff) and the opposing player will receive a payoff somewhere between the high and the low.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-actions: symmetric version (see section 3.2) actions in this game. Must be >= 1 and <= 500, but please note that using large numbers of actions will result in exponentially large games. Large numbers should not be used in games with more than two players.'};
	minGmActions = 2;
	maxGmActions = inf;

case 'LocationGame'
	defaultString = '-actions B -a C -b D -l E -c F -price_low G';
	gameDescription = {'Creates an instance of the two person Location Game based on Hotellings original model.' 'In this game there is a street of length l. Player one has a shop set up distance a from one end of the street and player 2 has a shop set up distance b from the other end. Customers are uniformly distributed along the street and the cost of getting a good from a shop to a home on the street is c times the distance. The players must pick a price at which to sell their goods in order to maximize their profit assuming that production is free and customers will 	always choose the shop for which the combined good price and transportation cost is smaller.' 'Profits may be scaled if normalization is used, but relations between the parameters will remain the same and are thus important.' 'Be very careful randomizing parameters in this game. If the cost of transporting goods is too high, it will always be a dominant strategy for both players to choose their highest action and the game will lose some of its intended interesting properties.' '' 'Game Parameters:' '-actions: symmetric version (see section 3.2)' '-a: distance between the location of player 1s store and his end of the street. Must fall between 0 and 1000.' '-b: distance between the location of player 2s store and his end of the street. Must fall between 0 and 1000.' '-l: length of the entire street. Must be >= a+b, but <= 1000' '-c: cost per unit of transporting the goods. Must fall between 1 and 100. See the above note on randomization and values of this parameter.' '-price_low: lowest price each player can choose. Must be > 0 and 1000. The highest price each player can choose will then be price low + actions - 1.'};
	minGmActions = 2;
	maxGmActions = inf;
	
case 'MajorityVoting'
	defaultString = '-players A -actions B';
	gameDescription = {'Creates an instance of the Majority Voting Game.' 'In this version of the Majority Voting Game, players utilities for each candidate (i.e. action) being declared the winner are arbitrary and it is possible that a player would be indifferent between two or more candidates.' 'If multiple candidates have the same number of votes and this number is higher than the number of votes any other candidate has, then the candidate with the lowest number is declared winner.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-players' '-actions: symmetric version (see section 3.2)'};
	minGmActions = 2;
	maxGmActions = inf;

case 'MinimumEffortGame',
	defaultString = '-players A -actions B -a C -b D -c E';
	gameDescription = {'Creates an instance of the Minimum Effort Game.' 'In this game, the payoff for a player is determined by a formula a + bM - cE where E is the players effort and M is the minimum effort of any player.' 'In this game, the payoff for a player is determined by a formula a + bM - cE where E is the players effort and M is the minimum effort of any player.' 'Game Parameters:' '-players' '-actions: symmetric version (see section 3.2)' '-a: constant a used in formula a + bM - cE. Should be between -100 and 100.' '-b: coefficient b used in formula a + bM - cE. Should be between 0 and 100.' '-c: coefficient used in formula a + bM - cE.' 'Should be between 0 and 100 but must be < b'};
	minGmActions = 2;
	maxGmActions = inf;

case 'RandomGame'
	defaultString = '-players A -actions B';
	gameDescription = {'Creates a game with the given number of players with payoffs distributed uniformly at random.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-players' '-actions: non-symmetric version (see section 3.2)'};
	minGmActions = 2;
	maxGmActions = inf;
	
case 'RandomZeroSum'
	defaultString = '-actions B';
	gameDescription = {'Creates a 2 player Zero Sum Game.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' 'Note that when normalization is used, there may be error in the last digits of the decimal payoffs resulting in a games which are occasionally not quite zero sum.' '' 'Game Parameters:' '-actions: non-symmetric version (see section 3.2)'};
	minGmActions = 2;
	maxGmActions = inf;
	
case 'RockPaperScissors'
	defaultString = '';
	gameDescription = {'Creates an instance of the Rock, Paper, Scissors Game.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' 'Game Parameters:' 'No parameters.'};
	minGmActions = 3;
	maxGmActions = 3;
	
case 'TravelersDilemma'
	defaultString = '-players A -actions B -reward C';
	gameDescription = {'Creates an instance of Travelers Dilemma game.' 'In order to make the game interesting, the parameters should be set up so that the reward is larger than one (but usually smaller than the number of actions).  When this holds, the unique Nash equilibrium will be the unsatisfying equilibrium in which everyone chooses the smallest dollar amount.' 'When randomization is used, the reward will automatically be chosen from somewhere in this range.' '' 'Game Parameters:' '-players:    the number of players. Must be >= 2 and <= 100.' '-actions:    number of actions.  All players must have the same number of actions in this game.  Must be >= 1 and may be as large as 500, but note that using large numbers of actions will result in exponentially large games.  For most games, the number of actions should be much smaller.' '-reward: the amount of the reward for the player who claims the lowest dollar amount.  Must be > 0 and <= 100.'};
	minGmActions = 2;
	maxGmActions = inf;

case 'TwoByTwoGame'
    defaultString = '-type C';
    gameDescription = {'Creates a game of two actions and two players of a given type according to Rappoports classification.' 'By default, all payoffs will fall in the range [-100, 100]. This range can be altered by setting the normalization options or using integer payoffs.' '' 'Game Parameters:' '-type: type of the 2x2 game in Rappoports classification, in [1,85]'};
    minGmActions = 2;
    maxGmActions = 2;
    
case 'WarOfAttrition'
	defaultString = '-actions B -valuation_low C -valuation_high D -decrement_low E -decrement_high F';
	gameDescription = {'Creates an instance of the War of Attrition. In a War of Attrition, two players are in a dispute over an object, and each chooses a time to concede the object to the other player. If both concede at the same time, they share the object. Each player has a valuation of the object, and each players utility is decremented at every time step.' 'Payoffs are based on the ranges of valuations and decrements provided. Although normalization may have the effect that the ranges of the payoffs will change, the ratio of the valuation amount to the decrement amount will still come into play.' '' 'Game Parameters:' '-actions: symmetric version (see section 3.2)' '-valuation low: lower bound on the players valuations for the item, should be between 10 and 1000.' '-valuation high: upper bound on the players valuations for the item. Must be >= valuation_low.' '-decrement_low: lower bound on the amount that the worth of the object to a player is decremented by at each time step. Note that each player has a different decrement value. Should be <= valuation_low, and between 1 and 100.' '-decrement_high: upper bound on the amount that the worth of the object to a player is decremented by at each time step. Must be >= decrement low.'};
	minGmActions = 2;
	maxGmActions = inf;

otherwise
	error_gui( 'Title', 'Error detected', 'Unknown game type selected' );
	defaultString = '';
	gameDescription = '';
end

% If the functions are needed, then add this description as well
if ( funcNeeded )
	functionsDescription = {'The common functions currently built into GAMUT are described below.' '	* ConcaveTableFunction:' 'Represents a general concave function as a table of points. Function is evaluated by looking up nearest point to the x value. No interpolation is done.' 'Function Parameters:' '-min: minimum of the function' '-max: maximum of the function' '-points: number of points in the table lookup' '' '* DecreasingWrapper:' 'Takes an increasing base function, and makes it descreasing by negating and shifting up.' 'Function Parameters:' '-base func: name of the function to wrap, must be increasing.' '-min: the minimum of the function' '-base params: parameters to be handed off to the base function, must be enclosed in [ ]' '' '* ExpFunction:' 'A function of the form f(x) = e^(alpha*x) + beta' 'Function Parameters:' 	'-alpha: multiplicative constant. Should be in the range 0 to 1. Defaults to 1.'	'-beta: additive term. Defaults to 0.' '' '* IncreasingPoly:' 'Represents an increasing polynomial. Coefficients and degree can either be specified explicitely,	or randomized to lie within given ranges.' 'Function Parameters:' '-degree: degree of the polynomial function.' '-coefs: coefficients of polynomial (should be a list of d+1 numbers where d is the degree), in the increasing order of degree.' '-coef min: lower bound on polynomial coefficients, used only for randomizing. Can be anywhere from -1000 to 1000 but defaults to -10.' '-coef max: upper bound on polynomial coefficients, used only for randomizing. Can be anywhere from -1000 to 1000 but defaults to 10.' '' '* IncreasingTableFunction:' 'Represents a general increasing function as a table of points. Function is evaluated by looking up nearest point to the x value. No interpolation is done.' 'Function Parameters:' '-min: minimum of the function' '-max: maximum of the function' '-points: number of points in the table lookup' ''	'* LogFunction:' 'A function of the form f(x) = alpha * ln( x + k) + beta. k is calculated automatically as 1 - dMin, where dMin is the lower bound on the domain.'	'Function Parameters:' '-alpha: multiplicative constant' '-beta: additive term' ''	'* PolyFunction:' 'Represents a general polynomial. Coefficients and degree can either be specified explicitely, or randomized to lie within given ranges.' 'Function Parameters:' '-degree: degree of the polynomial function.' '-coefs: coefficients of polynomial (should be a list of d+1 numbers where d is the degree), in the increasing order of degree.' '-coef min: lower bound on polynomial coefficients, used only for randomizing. Can be anywhere from -1000 to 1000 but defaults to -10.' '-coef max: upper bound on polynomial coefficients, used only for randomizing. Can be anywhere from -1000 to 1000 but defaults to 10.' '' '* TableFunction:' 'Represents a general function as a table of points. Function is evaluated by looking up nearest point to the x value. No interpolation is done.' 'Function Parameters:' '-min: minimum of the function' '-max: maximum of the function' '-points: number of points in the table lookup'};
else
	functionsDescription = '';
end
