function giga_wolf_ret = updatestrategy( varargin )

% Name: updatestrategy.m
% Location: MALT/agentClasses/@giga_wolf_agent/updatestrategy.m
% Purpose: Updates GIGA-WoLF's strategy
% Input Parameters:  giga_wolf_agent object
%                    number of game being played
%                    current iteration
%                    stage_cellList
% Output Parameters: giga_wolf_agent object with updated strategy

giga_wolf = varargin{1}; % agent being updated
nGame = varargin{2}; % Number of game being played
iteration = varargin{3}; % The current iteration
stage_cellList = varargin{4}; % Cell structure with actions and rewards from the stage

% Get the ID number of the agent and its number of actions
IDNumber = get( giga_wolf, 'idNumber' );
nActions = get( giga_wolf, 'actions' );
agntNumber = get( giga_wolf, 'number' );

% Set the opponent's number
if ( IDNumber == 1 )
	IDopp = 2;
else
	IDopp = 1;
end

% Get the actions and rewards played and obtained in this iteration
allActions = stage_cellList{1};
allRew = stage_cellList{2};
% Get the agent's individual action and reward
agntAction = allActions( IDNumber );
oppAction = allActions( IDopp );
agntRew = allRew( IDNumber );

% Generate a pure distribution,
% with a '1' at the action that was played in this iteration
pDist = generate_pure( agntAction, nActions );

% Generate the stepsize information
alpha = 0.602;
% This sets 'a' to allow eta_{1} to be 1 and decrease thereafter
a = 10;
eta_t = a/( giga_wolf.bigA + iteration)^alpha;

% Get the agent's strategy and his 'z' strategy
strategyCurr = returnstrategy( giga_wolf );
zCurr = get( giga_wolf, 'z' );

% Take the opponent's action -> generate the possible rewards for each of the agent's actions
% possR holds the *POSSIBLE* rewards for the agent, given the opponent's action
possR = zeros(1, nActions );
oppPure = generate_pure( oppAction, nActions );
for agntAct = 1:nActions
	% Generate a pure strategy
	agntPure = generate_pure( agntAct, nActions );
	if ( agntNumber == 1 )
		cStrat = [agntPure; oppPure];
	else
		cStrat = [oppPure; agntPure];
	end

	possR( agntAct ) = value( nGame, cStrat, agntNumber );
end

% Update the agent's strategy, using the stepsize and *POSSIBLE* rewards
strategyHat = strategyCurr + eta_t*possR;
% Project this strategy
strategyHat = gw_projection( strategyHat );

% Update the agent's 'z' distribution, using the stepsize and *POSSIBLE* rewards
zNew = zCurr + (1/3)*eta_t*possR;
% Project this strategy
zNew = gw_projection( zNew );

% Calculate delta using sum of squared differences
d_num = pdist( [zNew; zCurr] ); % Defaults to using EUCLIDEAN norm. Faster by not specifying it
d_denom = pdist( [zNew; strategyHat] );
if ( d_denom == 0 )
	delta = 1;
else
	delta = min(1, d_num/d_denom );
end

% Do an update of the agent's strategy
strategyNew = strategyHat + delta*( zNew - strategyHat );

% Return this strategy
giga_wolf_ret = giga_wolf;
allStrats = get( giga_wolf_ret, 'strategy' );
allStrats( agntNumber, : ) = strategyNew;
giga_wolf_ret = set( giga_wolf_ret, 'strategy', allStrats );
giga_wolf_ret = set( giga_wolf_ret, 'z', zNew );
