function session( varargin )

% Name: session.m
% Location: MALT/sessionClasses/@session/session.m
% Purpose:	Constructor class for the session class
%				Stores much of the information for running two agent, 
%				repeated game testing sessions
%           Creates a global variable, sessionObject, hence no output
% Input Parameters: No parameters - default created
%                   1 Parameter - object to copy
%                   6 Parameters -	Cell list of agent structures
%                                   Cell list of game structures
%                                   Cell list of metric objects
%                                   Parameter object
%                                   Cell list of data items being saved
%                                   Array list of how often to save the data items
% Output Parameters: Session object that updates the global parameter

% Link global variable and make it visible in this function
global SessionObject
% Create partial structure containing dummy variable
% This is needed, since SessionObject already exists as a global variable
SessionObject = struct( 'agentList', {'dummy'} );

switch nargin
case 0 % Create default session object
	SessionObject.agentList = {};
	SessionObject.gameList = {};
	SessionObject.metricList = {};
	SessionObject.parameter = {};
	SessionObject.dataItems = {};
	SessionObject.howOften = [];
	SessionObject.agentCombination = {};
	SessionObject.fileList = {};

	SessionObject = class( SessionObject, 'session' );
case 1
	if ( isa( varargin{1}, 'session' ) )
		SessionObject = varargin{1};
	else
		error( 'session.m: Input argument is not a session object' );
	end
case 6
% Check first argument is a cell of agent structures
	tAgentL = varargin{1};
	if ( ~isa( tAgentL, 'cell' ) )
		tAgentL = {tAgentL};
	end
	if ( ~isa( tAgentL{1}, 'struct' ) )
		error( 'session.m: Require first argument to be a cell of structures, containing agent details' );
	end
	if ( length( tAgentL ) == 1 )
		printOut( sprintf( '\nsession.m: Only single agent passed. Assume test is for selfplay' ) );
%		tAgentL(2) = tAgentL(1);
	else
		printOut( sprintf( 'session.m: %d agents passed', length( tAgentL ) ) );
	end
	SessionObject.agentList = tAgentL;

% Check second argument is a cell of game structures
	tGameL = varargin{2};
	if ( ~isa( tGameL, 'cell' ) )
		tGameL = {tGameL};
	end
	if ( isa( tGameL{1,1}, 'game' ) )
		% Do nothing - this means we are running using the cluster code
	elseif ( ~isa( tGameL{1, 1}, 'struct' ) )
		error( 'session.m: Require second argument to be a cell of structures, containing game details' );
	end
	printOut( sprintf( 'session.m: %d games passed', size( tGameL, 2 ) ) );
	SessionObject.gameList = tGameL;

% Check third argument is a cell of metric objects	
	tMetricL = varargin{3};
	if ( ~isa( tMetricL, 'cell' ) )
		tMetricL = {tMetricL};
	end
	if ( ~isa( tMetricL{1}, 'metric' ) )
		error( 'session.m: Require third argument to be a cell of metric objects' );
	end
	printOut( sprintf( 'session.m: %d metrics passed', length( tMetricL ) ) );
	SessionObject.metricList = tMetricL;
	
% Check fourth argument is a parameter object
	tParam = varargin{4};
	if ( ~isa( tParam, 'parameter' ) )
		error( 'session.m: Require fourth argument to be a parameter object' );
	end
	printOut( sprintf( 'session.m: 1 parameter passed' ) );
	SessionObject.parameter = tParam;

	tDataI = varargin{5};
	if ( ~isa( tDataI, 'cell' ) )
		tDataI = {tDataI};
	end
	SessionObject.dataItems = tDataI;
	printOut( sprintf( 'session.m: %d data items to save passed', length( varargin{5} ) ) );
	SessionObject.howOften = varargin{6};
	printOut( sprintf( 'session.m: %d snapshot schedules passed\n', length( varargin{6} ) ) );

	% Set the versus list of agent vs agent
	SessionObject.agentCombination = versusList( length( tAgentL ) );
	% Obtain the size of this list
	[nFlips, comb] = size( SessionObject.agentCombination );
	% Preallocate the array to store the list of file names
	SessionObject.fileList = cell(1, comb*size( tGameL, 2 ) );

	SessionObject = class( SessionObject, 'session' );

otherwise
	error_disp = sprintf( 'Valid arguments:\nNo arguments - default created\n1 Argument - object to copy\n6 Arguments -  Cell list of agent structures, Cell list of game structures, Cell list of metric objects, Parameter object, Cell list of data items being saved, Array list of how often to save the data items' );
	
	disp( error_disp );
	error( 'session.m: Incorrect number of input arguments' );
end
