function [savedFileList] = tournament( )

% Name: tournament.m
% Location: MALT/sessionClasses/GlobalFiles/tournament.m
% Purpose: Runs a series of tournaments between two agents, 
%				from a list of 2+ agents
% Input Parameters:	None - uses global session object (SessionObject)
% Output Parameters: Cell list of the file names where data objects saved

% Make the global variables visible
global SessionObject
global MALT_HOME

% Create file name for the SessionObject
fNameSession = create_file_name( '', '', '', 'session' );
% Save the session object
save( fNameSession, 'SessionObject' );

% Pick out the values of the fields we need from the SessionObject
agntsAll = SessionObject.agentList;
gamesAll = SessionObject.gameList;
paramObj = SessionObject.parameter;
dataItms = SessionObject.dataItems;
metricsAll = SessionObject.metricList;
agntsCmb = SessionObject.agentCombination;
filesSvd = SessionObject.fileList;
[flips totalCombinations] = size( agntsCmb );

% Get the parameter object field values we need
totalIterations = get( paramObj, 'totalIterations' );
settleIterations = get( paramObj, 'settle' );
nRuns = get( paramObj, 'runs' );

origGamesAll = gamesAll;

totGameSets = size( gamesAll, 2 ) * totalCombinations * nRuns * 2;
totGames = size( gamesAll, 2 );

% Loop over all games
for cGame = 1:totGames

	% Cell array of games is already precreated,
	% so we can just take them
	SessionObject = set( SessionObject, 'gameList', origGamesAll );
	gamesAll = origGamesAll;
	% The game information does not differ for the same game, at different runs
	% Hence, we just take the game in the first run, to get the non-payoff information
	gamePlayed = gamesAll{1, cGame};

	% Check that this is indeed a game object
	if ( ~isa( gamePlayed, 'game' ) )
		error( sprintf( 'tournament.m: Expected a game object, but found a %s object', class( gamePlayed ) ) );
	end

	% Obtain the maximum number of actions for any of the agents
	mActions = max( get( gamePlayed, 'totalActions' ) ); 

	for cAgntPair = 1: totalCombinations
		% Get the number of actions for the agent, from the game
		allActs = get( gamePlayed, 'totalActions' );
		maxActions = allActs(1);
		agnt1Struct = agntsAll{ agntsCmb(1, cAgntPair) };
		agnt1Struct.actions = allActs; % Update the number of actions
		agnt2Struct = agntsAll{ agntsCmb(2, cAgntPair) };
		agnt2Struct.actions = allActs; % Update the number of actions
		% We send through the total number of actions, since this is used in the agent instantiation

		% Obtain the displaynames for the two agents
		agnt1DispName = agnt1Struct.displayName;
		agnt2DispName = agnt2Struct.displayName;

		% Get the game name and file name info
		gameName = get( gamePlayed, 'name' );
		gameFileName = get( gamePlayed, 'fileName' );
		% Create a data object
		dt_obj = data( nRuns, gameName, dataItms, { agnt1DispName agnt2DispName } );

		% Create the statistics object
		statsObj = statistics( 2, nRuns );

		for cFlip = 1:2 % Agents start as row|column, then flip to column|row
	
			% Loop over all runs/instances 
			for cRun = 1:nRuns

				% Convert agent structure to the object -->
				% We also set the ID numbers for the agents
				agnt1 = createAgentObj( agnt1Struct );
				agnt1 = set( agnt1, 'idNumber', 1 );
				agnt2 = createAgentObj( agnt2Struct );
				agnt2 = set( agnt2, 'idNumber', 2 );
				printOut([' agent 1 name: ' get( agnt1, 'displayName' ) ' agent 2 name: ' get( agnt2, 'displayName' )]);

				% Make sure that agnt1.number = 1 and agnt2.number = 2 for the first flip			 
				% Swop these around for the second flip
				switch cFlip
				case 1
					agnt1 = set( agnt1, 'number', 1 );
					agnt2 = set( agnt2, 'number', 2 );
				case 2
					agnt1 = set( agnt1, 'number', 2 );
					agnt2 = set( agnt2, 'number', 1 );
				end % switch: cFlip
% I think the above flipping is okay and changes everything that needs to change

				% BEGIN: including metric information
				metricSavedData = cell( 1, length( metricsAll ) );
				for cMetCounter = 1:length( metricsAll )
						cMetObj = metricsAll{cMetCounter};
						% Returns the variable for saving that metric's data in
						metricSavedData( 1, cMetCounter ) = { metric_data_format( cMetObj, allActs ) };
				end % for: cMet
				% END: including metric information

				% We need to set the gameList to the format expected by other functions
				% That is, a single row of generators for each instance
				% This is for a single run and all games:
				SessionObject = set( SessionObject, 'gameList', origGamesAll( cRun, : ) );

				printOut( ['Run number ', num2str( cRun ), ' for ' agnt1DispName ' vs ' agnt2DispName ' for flip ' num2str( cFlip ) ' on game ' gameName] );
				% Create the relevant series_data object
				% We only need to store for (totalIterations - settleIterations)
				sd_data = chooseSeries( agnt1, agnt2, 2, maxActions, totalIterations - settleIterations, cRun );

				% Loop over all iterations
				for cIter = 1: totalIterations
				% Each agent chooses an action
					action1 = selectaction( agnt1, cGame );
					action2 = selectaction( agnt2, cGame );
					cActions = [action1; action2];
				
					% Generate pure strategies for those actions
					switch cFlip
					case 1
						cStratPure = [generate_pure( action1, mActions ); generate_pure( action2, mActions )];
						cStrats = [returnstrategy( agnt1 ); returnstrategy( agnt2 )];
					case 2
						cStratPure = [generate_pure( action2, mActions ); generate_pure( action1, mActions )];
						cStrats = [returnstrategy( agnt2 ); returnstrategy( agnt1 )];
					end % switch: cFlip			

					% Obtain rewards
					reward1 = value( cGame, cStratPure, get( agnt1, 'number' ) );
					reward2 = value( cGame, cStratPure, get( agnt2, 'number' ) );
					cRewards = [reward1; reward2];
						
					if ( cIter > settleIterations )
					% We only store the iterations after the settling in has occured
						storeIteration = cIter - settleIterations;
						% We do not need to worry about flipping the things stored, since
						% the values are calculated with the flipped agent numbers, but
						% everything is still saved according to the original order of the agents
						% Create temp, blank variables, for those not previously created
						cExp = [];
						estStrat = [];
						actStrat = [];
						
						% Loop over all the data items that we need to store
						for snaps = 1:length( dataItms )
							switch lower( dataItms{snaps} )
							case 'rewards'
								sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cRewards ); 
							case 'strategies'
								sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cStrats );
							case 'actionstaken'
								sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cActions );
							case 'expectedvalue'
								exp1 = value( cGame, cStrats, get( agnt1, 'number' ) );
								exp2 = value( cGame, cStrats, get( agnt2, 'number'  ) );
								cExp = [exp1; exp2];
								sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cExp );
							case 'trackbeliefs'
								estStratByAgnt1 = return_tracked_strategy( agnt1 );
								actStratForAgnt2 = returnstrategy( agnt2 );
								estStratByAgnt2 = return_tracked_strategy( agnt2 );
								actStratForAgnt1 = returnstrategy( agnt1 );
								estStrat = [estStratByAgnt1; estStratByAgnt2];
								actStrat = [actStratForAgnt2; actStratForAgnt1];
								sd_data = add_to_series_data( sd_data, storeIteration, 'trackbeliefs', estStrat );
								sd_data = add_to_series_data( sd_data, storeIteration, 'actualStrats', actStrat );								
							end % switch: lower( dataItms{snaps} )
						end % for: snaps

						% Create data structure with this information
						dataStructure = struct( 'rewards', cRewards, 'strategies', cStrats,...
														'actions', cActions, 'expectedvalue', cExp,...
														'trackEst', estStrat, 'trackActual', actStrat );
						% Loop over all metrics, running each one on this data
						for cMetCounter = 1:length( metricsAll )
							cMetObj = metricsAll{cMetCounter};
							% Gets the current data for that metric
							availMetData = metricSavedData{ 1, cMetCounter };
							availMetData = run_metric_single( cMetObj, availMetData, dataStructure, agnt1, agnt2, cGame, cRun, storeIteration );
							metricSavedData( 1, cMetCounter ) = {availMetData};
						end % for: cMet
					end % if: cIter > settleIterations
						
					% Now update the strategy of both agents and possibly the game as well.
					% We input the number of the game and then the updatestrategy method takes what is needed from the global SessionObject
					% If the game is updated, then it will be set back into the SessionObject

					agnt1 = updatestrategy( agnt1, cGame, cIter, {cActions cRewards} );
					agnt2 = updatestrategy( agnt2, cGame, cIter, {cActions cRewards} );
					% Update the beliefs of each agent
					agnt1 = updatebeliefs( agnt1, cActions );
					agnt2 = updatebeliefs( agnt2, cActions );
				end % for: cIter = 1:totalIterations
				
				% Create blank structure
				metStatStruct = struct;
				% Get the display names of the agents
				agntNames = { get( agnt1, 'displayName' ) get( agnt2, 'displayName' ) };
				for cMetCounter = 1:length( metricsAll )
					cMetObj = metricsAll{cMetCounter};
					metName = get( cMetObj, 'name' );
					% Gets the current data for that metric
					availMetData = metricSavedData{ 1, cMetCounter }; 
					% Clean up the stored metric data into a metric statistic object
					metStatObj = metric_cleanup( cMetObj, availMetData, gameName, agntNames );
					% Add this metric statistic object to the structure of metric statistic objects
					metStatStruct = setfield( metStatStruct, metName, metStatObj );
				end % for: cMet
				% Add this structure to the statistics object
				statsObj = add_to_statistics( statsObj, metStatStruct, cFlip, cRun );
				% Add the series_data object to the data object
				dt_obj = add_to_data( dt_obj, sd_data, cFlip );
				% Calculate the run number out of a maximum
				runN = (cGame-1)*(nRuns*2*totalCombinations) + (cAgntPair-1)*(nRuns*2)+(cFlip-1)*nRuns+cRun;
				printOut( ['      Completed ' int2str(runN) ' out of ' int2str( totGameSets ) ' runs'] );
			end % for: runs
		end % for: flips

		% For the statistics object/file values: Get a file name
		fNameStats = create_file_name( agnt1DispName, agnt2DispName, gameFileName(1:(end-2)), 'stats' );

		fNameFile = create_file_name( '', '', '', 'fileList' );
		% For the data object/file values: Get a file name
		fNameData = create_file_name( agnt1DispName, agnt2DispName, gameFileName(1:(end-2)), 'data' );

		% Save the file name to the list of file names
		filesSvd( (cGame-1)*totalCombinations + cAgntPair ) = {fNameData}; % Save the file name to the correct position
		% Update the filelist to the SessionObject
		SessionObject = set( SessionObject, 'fileList', filesSvd );

		% Save the files
		save( fNameStats, 'statsObj' ); % Save the statistics object to a file
		save( fNameData, 'dt_obj' ); % Save the data object to a file
		save( fNameFile, 'filesSvd' ); % Save the filelist to a file

	end % for: agent combinations
end % for: games

% Set the SessionObject game list back to its original state
SessionObject = set( SessionObject, 'gameList', origGamesAll );

% Save the list of file names
saveFileList = filesSvd;
% Save the SessionObject
save( fNameSession, 'SessionObject' );
