function tournament_cluster( gameNumb, agnt1Numb, agnt2Numb, runNumb, flipNumb )

% Name: tournament_cluster.m
% Location: MALT/sessionClasses/GlobalFiles/tournament_cluster.m
% Purpose: Runs a single tournament between two agents, 
%          as part of a full MALT test run. 
%          This is normally used in cluster environments
% Input Parameters:  game number
%                    agent 1 number
%                    agent 2 number
%                    run number
%                    flip number
% Output Parameters: Nothing 

% Make all global variables visible
global SessionObject
global MALT_HOME
global STORAGE_DIRECTORY
global SESSION_NAME

session; % Create a dummy SessionObject variable

% First we want to load the SessionObject
sessStruct = load( [MALT_HOME STORAGE_DIRECTORY '/' SESSION_NAME '/session.mat'] );
SessionObject = sessStruct.SessionObject;

% Pick out the details we need from the SessionObject
agntsAll = SessionObject.agentList;
gamesAll = SessionObject.gameList;
metricsAll = SessionObject.metricList;
paramObj = SessionObject.parameter;
dataItms = SessionObject.dataItems;

% Obtain the values of fields from the parameter object
totalIterations = get( paramObj, 'totalIterations' );
settleIterations = get( paramObj, 'settle' );

% Take the correct game
gamePlayed = gamesAll{ runNumb, gameNumb};
gameName = get( gamePlayed, 'name' );

% We now want to set the SessionObject.gameList to be a vector of one run across across games.
% This ensures that functions that use a single number, 'nGame' to identify the game,
% do so correctly.
gamesAll = gamesAll( runNumb, : );
SessionObject = set( SessionObject, 'gameList', gamesAll );

% Check that this is definitely a game object
if ( ~isa( gamePlayed, 'game' ) )
	error( sprintf( 'tournament.m: Expected a game object, but found a %s object', class( gamePlayed ) ) );
end

% Then take the correct agents and set their number of actions
allActs = get( gamePlayed, 'totalActions' );
maxActions = allActs(1);
agnt1Struct = agntsAll{ agnt1Numb };
agnt1Struct.actions = allActs;
agnt2Struct = agntsAll{ agnt2Numb };
agnt2Struct.actions = allActs;

% Convert agent structure to object
% Also set the ID numbers for the agents
agnt1 = createAgentObj( agnt1Struct );
agnt1 = set( agnt1, 'idNumber', 1 );
agnt2 = createAgentObj( agnt2Struct );
agnt2 = set( agnt2, 'idNumber', 2 );
% Printout these values for reference
printOut( [' agent 1 name: ' get( agnt1, 'displayName' ) ', agent 2 name: ' get( agnt2, 'displayName' ) ', on game ' get( gamePlayed, 'fileName' ) ', for run number: ' num2str( runNumb ) ', flip: ' num2str( flipNumb )] );
% Get the display names for the agents
agnt1DispName = get( agnt1, 'displayName' );
agnt2DispName = get( agnt2, 'displayName' );

% Get the file name for the game
gameFileName = get( gamePlayed, 'fileName' );
% Create a file name for the cluster job
sdFileName = create_file_name_cluster( agnt1DispName, agnt2DispName, gameFileName, '', flipNumb, 'series_data' );

% Create file name for the "metric_data" file resulting from the cluster job
metDataFileName = create_file_name_cluster( agnt1DispName, agnt2DispName, gameFileName, '', flipNumb, 'metric_data' );

% Preallocate space for the saved metric values
metricSavedData = cell( 1, length( metricsAll ) );
% Loop over all metrics
for cMetCounter = 1:length( metricsAll )
	cMetObj = metricsAll{cMetCounter}; % Get the current metric
	% Returns the variable for saving the metrics data in
	metricSavedData( 1, cMetCounter ) = { metric_data_format( cMetObj, allActs ) };
end % for: cMet

% Make sure that agnt1.number = 1 and agnt2.number = 2 for the first flip
% Swop these around for the second flip
switch flipNumb
case 1
	agnt1 = set( agnt1, 'number', 1 );
	agnt2 = set( agnt2, 'number', 2 );
case 2
	agnt1 = set( agnt1, 'number', 2 );
	agnt2 = set( agnt2, 'number', 1 );
end % switch: flipNumb

% Create the relevant series_data object
% We only need to store for (totalIterations - settleIterations)
sd_data = chooseSeries( agnt1, agnt2, 2,maxActions, totalIterations - settleIterations, runNumb );

% Obtain the maximum number of actions for any of the agents
mActions = max( get( gamePlayed, 'totalActions' ) ); 

% Loop over all the iterations
for cIter = 1: totalIterations
	% Each agent chooses an action
	action1 = selectaction( agnt1, gameNumb );
	action2 = selectaction( agnt2, gameNumb );
	cActions = [action1; action2];

   % Generate pure strategies for those actions
	% Put those strategies into a strategies matrix
	switch flipNumb
	case 1
		cStratPure = [generate_pure( action1, mActions ); generate_pure( action2, mActions )];
		cStrats = [returnstrategy( agnt1 ); returnstrategy( agnt2 )];
	case 2
		cStratPure = [generate_pure( action2, mActions ); generate_pure( action1, mActions )];
		cStrats = [returnstrategy( agnt2 ); returnstrategy( agnt1 )];
	end % switch: flipNumb

	% Obtain the relevant payoffs
	reward1 = value( gameNumb, cStratPure, get( agnt1, 'number' ) );
	reward2 = value( gameNumb, cStratPure, get( agnt2, 'number' ) );
	cRewards = [reward1; reward2];
					
	if ( cIter > settleIterations )
		% We only store the iterations after the settling in has occured
		storeIteration = cIter - settleIterations;

      % Create temp, blank variables, for those not previously created
      cExp = [];
		estStrat = [];
		actStrat = [];

		% We do not need to worry about flipping the things stored, since
		% the values are calculated with the flipped agent numbers, but
		% everything is still saved according to the original order of the agents
		% Loop over the possible data items and store what is relevant
		for snaps = 1:length( dataItms )
			switch lower( dataItms{snaps} )
			case 'rewards'
				sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cRewards ); 
			case 'strategies'
				sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cStrats );
			case 'actionstaken'
				sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cActions );
			case 'expectedvalue'
				exp1 = value( gameNumb, cStrats, get( agnt1, 'number' ) );
				exp2 = value( gameNumb, cStrats, get( agnt2, 'number'  ) );
				cExp = [exp1; exp2];
				sd_data = add_to_series_data( sd_data, storeIteration, dataItms{snaps}, cExp );
			case 'trackbeliefs'
            estStratByAgnt1 = return_tracked_strategy( agnt1 );
				actStratForAgnt2 = returnstrategy( agnt2 );
				estStratByAgnt2 = return_tracked_strategy( agnt2 );
				actStratForAgnt1 = returnstrategy( agnt1 );
				estStrat = [estStratByAgnt1; estStratByAgnt2];
				actStrat = [actStratForAgnt2; actStratForAgnt1];
				sd_data = add_to_series_data( sd_data, storeIteration, 'trackbeliefs', estStrat );
				sd_data = add_to_series_data( sd_data, storeIteration, 'actualStrats', actStrat );
			end % switch: lower( dataItms{snaps} )
		end % for: snaps

		% Create a data structure
      dataStructure = struct( 'rewards', cRewards, 'strategies', cStrats,...
										'actions', cActions, 'expectedvalue', cExp,...
										'trackEst', estStrat, 'trackActual', actStrat );
		% Loop over all metrics and preallocate them
		for cMetCounter = 1:length( metricsAll )
			cMetObj = metricsAll{cMetCounter};
			% Gets the current data for that metric
			availMetData = metricSavedData{ 1, cMetCounter };
			availMetData = run_metric_single( cMetObj, availMetData, dataStructure, agnt1, agnt2, gameNumb, runNumb, storeIteration );
			metricSavedData( 1, cMetCounter ) = { availMetData };
		end % for: cMet
	end % if: cIter > settleIterations

	% Now update the strategy of both agents and possibly the game as well.
	% We input the number of the game and then the updatestrategy method takes what is needed from the global SessionObject
	% If the game is updated, then it will be set back into the SessionObject

	agnt1 = updatestrategy( agnt1, gameNumb, cIter, {cActions cRewards} );
	agnt2 = updatestrategy( agnt2, gameNumb, cIter, {cActions cRewards} );
	% Update the beliefs of the two agents
	agnt1 = updatebeliefs( agnt1, cActions );
	agnt2 = updatebeliefs( agnt2, cActions );
end % for: cIter = 1:totalIterations

% Create an empty structure
metStatStruct = struct;
% Get the display names forthe two agents
agntNames = { get( agnt1, 'displayName' ) get( agnt2, 'displayName' ) };
% Loop over the metrics
for cMetCounter = 1:length( metricsAll )
	cMetObj = metricsAll{cMetCounter};
	metName = get( cMetObj, 'name' );
	% Gets the current data for that metric
	availMetData = metricSavedData{ 1, cMetCounter };
	% Clean up the stored metric data into a metric statistic object
	metStatObj = metric_cleanup( cMetObj, availMetData, gameName, agntNames );
	% Add this metric statistic object to the structure of metric statistic objects
	metStatStruct = setfield( metStatStruct, metName, metStatObj );
end % for: cMet

% Since this is the cluster version of everything, we don't add this to the statistics file,
% we just save the structure

save( metDataFileName, 'metStatStruct' );

% We should now save the series_dat object to a file
% Currently this is turned off (space saver)
% Save the series_data object
%save( sdFileName, 'sd_data' )
