function [plot_dataStruct barVsl] = agg_visual_final( barVsl, plot_data, condAgg, condFree, freeNames, nFree, val4dFree, errorbarStatus, nAllVars, whichAgents )

% Name: agg_visual_final.m
% Location: MALT/visualClasses/@bar_visual/agg_visual_final.m
% Purpose:  Do the final aggregation for the visual object on the data
%           What this final step is, is dependent on the aggregation method,
%           whether error bars are wanted, etc.
%
% Input Parameters: 8 Parameters - visual object (bar_visual)
%                                  pre-final set of data
%                                  type of aggregation wanted
%                                  name of the selected free variable(s)
%                                  array of labels within each free variable
%                                  number of elements in the selected free variable(s)
%                                  array containing values corresponding to the dimensions of the 4d matrix that correspond to the free variables
%                                  error bar status - on (1) or off (0)
%                                  number of elements in each of agents, games, run and iterations
%                                  number indicating whether using agents or row||column agents
%
% Output Parameters: Plot data structure, containing all the data,
%                       that is needed by this visualisation object
%                    Bar visual object (possibly updated)

%if find( strcmp( 'iterations', lower( condFree ) ) )
if ~errorbarStatus
	% We don't have error bars - we know that this has to be a 2d matrix
	% In other visual object, we may still have a 4d matrix, for 'none', 
	% but in bar_visual, we know this now must be a 2d matrix

	if find( strcmp( 'mean', lower( condAgg ) ) )
		% If the aggregation method is 'mean'
		% How much do we divide by?
		% This is the total number of free variables multiplied together,
		% divided by the product of the selected free variables!
		totalDivision = ( nAllVars{1}*nAllVars{2}*nAllVars{3}*nAllVars{4}*nAllVars{5}*(whichAgents+1) ) / prod( nFree );
		% Make it the average
		plot_data = plot_data/totalDivision; 
	% There is no need for an else, since 'total' doesn't change plot_data
	end

	if ( length( nFree ) > 1 )
		% ie. we have two selected free variables
		if ( isequal( lower( condFree{2} ), 'row agent' ) || isequal( lower( condFree{2} ), 'agent' ) )
			% The agent is being plotted as the 2nd free variable, but this
			% means that it would be drawn as the 'x-axis' variables, which
			% we don't want - thus we want to transpose the data matrix,
			% to ensure that it gets plotted as the legends
			% or at least that was the original thinking...
			% In fact, nothing needs to change, the line below does nothing
			plot_data;

		end
	end
	% Create a structure with the plot_data
	plot_dataStruct = struct( 'data', plot_data );
else
	% errorbars are being used!
	% We then know that iterations isn't one of the free variables
	% plot_data will be 4d matrix

	if find( strcmp( 'mean', lower( condAgg ) ) )
		% Aggregation method is mean - need to divide by the number of iterations
		% So we divide the 4d matrix by # iterations
		% Each element is now the average of the game
		plot_data = plot_data/( ( whichAgents + 1 )*nAllVars{5} );
		% if the aggregation method is total, then we don't need to change plot_data
	end
		
	% We have been saving rows to the second aggregation index, as the first
	% aggregation is the larger one and we want this along the columns
	if ( length( nFree ) == 2 ) % Two free variables
		% We need to keep track of the lowerdata and upperdata split & median
		medianData = zeros( nFree(2), nFree(1) ); 
		lowerData = zeros( nFree(2), nFree(1) );
		upperData = zeros( nFree(2), nFree(1) );
		
		for agg1Count = 1:nFree(1) % Over first free variable
			returnDataMid = getDataPortion( plot_data, val4dFree(1), agg1Count );
			for agg2Count = 1:nFree(2) % Over second free variable
				returnData = getDataPortion( returnDataMid, val4dFree(2), agg2Count );
				% Turn into a vector and sort it, for use in errorbar
				returnData = sort( returnData(:) );
				nDat = length( returnData );
				% Now do the calculations for the errorbars
				% Currently correspond to median, 1st and 3rd percentile
				if mod( nDat, 2 )
					% Odd number of elements
					q = (nDat+1)/2;
					medianData( agg2Count, agg1Count ) = returnData( q );
				else
					% Even number of elements
					q = nDat/2;
					medianData( agg2Count, agg1Count ) = ( returnData( q ) + returnData( q + 1 ) )/2;
				end
				if mod( q, 2 )
					% q is odd
					r = (q + 1)/2;
					lowerData( agg2Count, agg1Count ) = returnData( r );
					upperData( agg2Count, agg1Count ) = returnData( nDat+1-r );
				else
					% q is even
					r = q/2;
					lowerData( agg2Count, agg1Count ) = ( returnData( r ) + returnData( r+1 ) )/2;
					upperData( agg2Count, agg1Count ) = ( returnData( nDat+1-r ) + returnData( nDat-r ) )/2;
				end % lowerData, upperData calculations
			end % for: agg2Count
		end % for: agg1Count

	else
		% Only one free variable
		% Preallocate space for the median and lower and upper data splits
		medianData = zeros( 1, nFree(1) ); 
		lowerData = zeros( 1, nFree(1) );
		upperData = zeros( 1, nFree(1) );
		
		% Over all of the elements in the free variable
		for agg1Count = 1:nFree(1)
			returnData = getDataPortion( plot_data, val4dFree(1), agg1Count );
			returnData = sort( returnData(:) );
			% Now do the errorbar calculations
			nDat = length( returnData );
			% Now do the calculations for the errorbars
			% Currently correspond to median, 1st and 3rd percentile
			if mod( nDat, 2 )
				% Odd number of elements
				q = (nDat+1)/2;
				medianData( 1, agg1Count ) = returnData( q );
			else
				% Even number of elements
				q = nDat/2;
				medianData( 1, agg1Count ) = ( returnData( q ) + returnData( q + 1 ) )/2;
			end
			if mod( q, 2 )
				% q is odd
				r = (q + 1)/2;
				lowerData( 1, agg1Count ) = returnData( r );
				upperData( 1, agg1Count ) = returnData( nDat+1-r );
			else
				% q is even
				r = q/2;
				lowerData( 1, agg1Count ) = ( returnData( r ) + returnData( r+1 ) )/2;
				upperData( 1, agg1Count ) = ( returnData( nDat+1-r ) + returnData( nDat-r ) )/2;
			end % lowerData, upperData calculations
		end % for: agg1Count
	end % if: length( nFree )

	if ( length( nFree ) > 1 )
		% ie. we have two selected free variables
		if ( isequal( lower( condFree{2} ), 'row agent' ) || isequal( lower( condFree{2} ), 'agent' ) )
			% We do not do anything
		end
	end

	% Create the structure storing the plot data
	plot_dataStruct = struct( 'medianData', medianData, 'lowerData', lowerData, 'upperData', upperData );
end
