function [dataStore barVsl] = plot_data_format( barVsl, freeLengths, freeNames, condFree, aggMethod, errorbarStatus, lengthValues, metricObj )

% Name: plot_data_format.m
% Location: MALT/visualClasses/@bar_visual/plot_data_format.m
% Purpose: Creates the data structure where all of the data will be stored,
%          This is the data from a metric, that will be visualised
% Input Parameters:  5 Parameters - bar visual object
%                                   array of lengths of free variables
%                                   array of labels within each free variable
%                                   names of free variables
%                                   method of aggregation on the non-selected free variables
%                                   array of length values for agents, games, runs
%                                   metric object
% 
% Output Parameters: data store variable, for storing the data
%                    updated bar visual object

if ( length( freeLengths ) == 2)
	% Two free variables selected
	if find( strcmp( lower( condFree ), 'iterations' ) )
		% If 'iterations' is one of the free variables, 
		% then just create a 2d matrix, to store the data.
		% In this case, we know we won't have any errorsbars, 
		% since they are disallowed when 'iterations' is a free variable
		dataStore = zeros( freeLengths(2), freeLengths(1) );
	else
		% We now may have the possibility of errorbars, so we check
		if errorbarStatus
			% We do have errorbars, thus need to create dataStore as a 4d
			% This is referenced as (agnt1, agnt2, game, row)
			% OR if we aren't aggregating at all - in which case we need 4d as well
			dataStore = zeros( lengthValues{1}, lengthValues{2}, lengthValues{3}, lengthValues{4} );
		else
			% No errorbars, 
			% No 'iterations' as a selected free variable, thus dataStore
			% is just a 2d matrix of the lengths of the free variables
			dataStore = zeros( freeLengths(2), freeLengths(1) );
		end
	end
else
	% One free variables selected
	if ( errorbarStatus )
		% Using error bars, need 4d matrix
		dataStore = zeros( lengthValues{1}, lengthValues{2}, lengthValues{3}, lengthValues{4} );
	else
		dataStore = zeros( 1, freeLengths(1) );
	end
end

changeUp = 0; % Hack variable for the title creation
if ( length( condFree ) == 1 )
	% Only one free variable selected
	cLabelNames = get( barVsl, 'gcaTitles' );
	% Set values for the x-axis-tick labels
	newLabelNames = {cLabelNames{1:end} 'XTickLabel' 'XTick'};
	barVsl = set( barVsl, 'gcaTitles', newLabelNames );
	cLabels = get( barVsl, 'gcaLabels' );
	% Create the new labels
	newLabels = {cLabels{1:end} freeNames{1} [1:length( freeNames{1} )]};
	barVsl = set( barVsl, 'gcaLabels', newLabels );
	% Set up the legend
	barVsl = set( barVsl, 'legend', '' );
	% Set up the x-axis labels
	barVsl = set( barVsl, 'labelX', condFree{1} );
	xName = condFree{1};
	legName = '';
	% Set up the y-axis labels
	barVsl = set( barVsl, 'labelY', get( metricObj, 'displayName' )  );
else
	% Two free variables selected
	% Set the xticks title
	if ( length( freeNames{2} ) == 1 )
		% Only a single value in the 2nd selected free variable
		% so almost equivalent to the one free variable selected case
		cLabelNames = get( barVsl, 'gcaTitles' );
		newLabelNames = {cLabelNames{1:end} 'XTickLabel' 'XTick'};
		barVsl = set( barVsl, 'gcaTitles', newLabelNames );
		cLabels = get( barVsl, 'gcaLabels' );
		if ( isequal( lower( condFree{2} ), 'row agent' ) || isequal( lower( condFree{2} ), 'agent' ) )
			% The agent is being plotted as the 2nd free variable, but this
			% means that it would be drawn as the 'x-axis' variables, which
			% we don't want - thus we want to transpose the data matrix,
			% to ensure that it gets plotted as the legends
			newLabels = {cLabels{1:end} freeNames{1} [1:length( freeNames{1} )]};
			barVsl = set( barVsl, 'gcaLabels', newLabels );
			barVsl = set( barVsl, 'legend', freeNames{2} ); % Set up the legend	
			barVsl = set( barVsl, 'labelX', condFree{1} ); % Set up the x-axis label
		else
			newLabels = {cLabels{1:end} freeNames{1} [1:length( freeNames{1} )]};
			barVsl = set( barVsl, 'gcaLabels', newLabels );
			barVsl = set( barVsl, 'legend', freeNames{2} ); % Set up the legend	
			barVsl = set( barVsl, 'labelX', condFree{1} ); % Set up the x-axis label
		end
		% This is slightly reversed from the normal method of assigning names,
		% but this is to take care of the title correctly
		xName = condFree{2};
		legName = condFree{1};
		changeUp = 1;
		barVsl = set( barVsl, 'labelY', get( metricObj, 'displayName' )  );
	else
	% Two free variables selected
	% Second free variable has > 1 element in it
		barVsl = set( barVsl, 'labelY', get( metricObj, 'displayName' ) );

		cLabelNames = get( barVsl, 'gcaTitles' );
		barVsl = set( barVsl, 'gcaTitles', {cLabelNames{1:end} 'XTickLabel' 'XTick'} );
		cLabels = get( barVsl, 'gcaLabels' );

		% Agent is first free variable, runs along columns of plot_data matrix,
		% thus need to plot it as the legend, since 'bar' plots by rows
		newLabels = {cLabels{1:end} freeNames{2} [1:length( freeNames{2} ) ]};
		barVsl = set( barVsl, 'gcaLabels', newLabels );
		barVsl = set( barVsl, 'legend', freeNames{1} ); % Set up the legend
		barVsl = set( barVsl, 'labelX', condFree{2} );

		xName = condFree{2};
		legName = condFree{1};
	end
end

% Set the title in the bar visual object
barVsl = set( barVsl, 'title', create_title( barVsl, metricObj, aggMethod, xName, legName, changeUp ) );
