function [dataStore ptVsl] = plot_data_format( ptVsl, freeLengths, freeNames, condFree, aggMethod, errorbarStatus, lengthValues, metricObj )

% Name: plot_data_format.m
% Location: MALT/visualClasses/@plot_visual/plot_data_format.m
% Purpose: Creates the data structure where all of the data will be stored,
%          This is the data from a metric, that will be visualised
% Input Parameters:  5 Parameters - plot visual object
%                                   array of lengths of free variables
%                                   array of labels within each free variable
%                                   names of free variables
%                                   method of aggregation on the non-selected free variables
%                                   array of length values for agents, games, runs
%                                   metric object
% 
% Output Parameters: data store variable, for storing the data
%                    updated plot visual object

if ( length( condFree ) == 2 )
	% Two free variables selected
	if find( strcmp( lower( condFree ), 'iterations' ) )
		% If 'iterations' is one of the free variables, 
		% then just create a 2d matrix, to store the data.
		% In this case, we know we won't have any errorsbars, 
		% since they are disallowed when 'iterations' is a free variable
		dataStore = zeros( freeLengths(2), freeLengths(1) );
		% This is since we want our largest free variable to be the columns
	else
		% We now may have the possibility of errorbars, so we check
		if ( errorbarStatus )
			% We do have errorbars, thus need to create dataStore as a 4d
			% This is referenced as (agnt1, agnt2, game, row)
			% OR if we aren't aggregating at all - in which case we need 4d as well
			% So preallocation is a 4D matrix
			dataStore = zeros( lengthValues{1}, lengthValues{2}, lengthValues{3}, lengthValues{4} );
		else
			% No errorbars
			% No 'iterations' as a free variable, thus dataStore is just
			% a 2d matrix of the lengths of the free variables
			% We can preallocate this matrix
			dataStore = zeros( freeLengths(2), freeLengths(1) );		
		end
	end
else
	% One free variables selected
	if ( errorbarStatus )
		% Using error bars, need 4d matrix
		dataStore = zeros( lengthValues{1}, lengthValues{2}, lengthValues{3}, lengthValues{4} );
	else
		dataStore = zeros( 1, freeLengths(1) );
	end
end

% Set the y-label
ptVsl = set( ptVsl, 'labelY', get( metricObj, 'displayName' ) ); % Set y-axis label

% Set the xtick and xticklabel title
cLabels = get( ptVsl, 'gcaTitles' );
newLabelNames = {cLabels{1:end} 'XTickLabel' 'XTick'};
ptVsl = set( ptVsl, 'gcaTitles', newLabelNames );
cLabels = get( ptVsl, 'gcaLabels' ); % get the label names

if ( length( condFree ) == 2 )
	% Check if agents are being used as first free variable
	if ( ( isequal( lower( condFree{1} ), 'row agent' ) || isequal( lower( condFree{1} ), 'agent' ) ) && ( length( freeNames{1} ) == length( freeNames{2} ) ) )
		% Just a slight title/legend change,
		% because of how plot and the system handles things
		ptVsl = set( ptVsl, 'labelX', condFree{2} );
		newLabels = {cLabels{1:end} freeNames{2} [1:length( freeNames{2} )]};
		ptVsl = set( ptVsl, 'gcaLabels', newLabels );
		% Setup the legend
		ptVsl = set( ptVsl, 'legends', freeNames{1} );
	else
		% Set the X labels
		ptVsl = set( ptVsl, 'labelX', condFree{1} ); % Set up the x-axis label
		if isequal( lower( condFree{1} ), 'iterations' )
			newLabels = {cLabels{1:end} '' [] };
		else
			newLabels = {cLabels{1:end} freeNames{1} [1:length( freeNames{1} )]};
		end
		ptVsl = set( ptVsl, 'gcaLabels', newLabels );
		ptVsl = set( ptVsl, 'legends', freeNames{2} );
	end
else
	ptVsl = set( ptVsl, 'labelX', '' ); % Set up the x-axis label
		if isequal( lower( condFree{1} ), 'iterations' )
			newLabels = {cLabels{1:end} '' [] };
		else
			newLabels = {cLabels{1:end} freeNames{1} [1:length( freeNames{1} )]};
		end
		ptVsl = set( ptVsl, 'gcaLabels', newLabels );
		ptVsl = set( ptVsl, 'legends', '' ); % There is no legends, since no second free var selected
end
