function [plot_dataStruct stVsl] = agg_visual_final( stVsl, plot_data, condAgg, condFree, freeNames, nFree, val4dFree, errorbarStatus, nAllVars, whichAgents )

% Name: agg_visual_final.m
% Location: MALT/visualClasses/@scatter_visual/agg_visual_final.m
% Purpose:  Do the final aggregation for the visual object on the data
%           What this final step is, is dependent on the aggregation method,
%           whether error bars are wanted, etc.
%
% Input Parameters: 8 Parameters - visual object (scatter_visual)
%                                  pre-final set of data
%                                  type of aggregation wanted
%                                  name of the selected free variable(s)
%                                  array of labels within each free variable
%                                  number of elements in the selected free variable(s)
%                                  array containing values corresponding to the dimensions of the 4d matrix that correspond to the free variables
%                                  error bar status - on (1) or off (0)
%                                  number of elements in each of agents, games, run and iterations
%                                  number indicating whether using agents or row||column agents
%
% Output Parameters: Plot data structure, containing all the data,
%                       that is needed by this visualisation object
%                    Scatter visual object (possibly updated)

finalL = {};

if ( length( condFree ) == 1 )
	% Single free variable selected
	% Obtain the total size of the arryas
	arraySizes = ( nAllVars{1}*nAllVars{2}*nAllVars{3}*nAllVars{4} )/nFree;

	% Preallocate space for the matrices to store the x- and y- values
	finalX = zeros(nFree, arraySizes );
	finalY = zeros(nFree, arraySizes );

	allNames = freeNames{1};
	for agg1Count = 1:nFree(1)
		returnDataX = getDataPortion( plot_data.xData, val4dFree(1), agg1Count );
		% Divide by the number of iterations to obtain the average of this
		% The x-axis is untouched, since there will be multiple scatter "lines"
		returnDataX = returnDataX(:);
		finalX( agg1Count, : ) = returnDataX;

		% Get the relevant y value of the points
		returnDataY = getDataPortion( plot_data.yData, val4dFree(1), agg1Count );
		returnDataY = returnDataY(:)/( nAllVars{5}*( whichAgents+1 ) );
		finalY( agg1Count, : ) = returnDataY;
	end
else
	% *** Two free variables selected ***
	arraySizes = ( nAllVars{1}*nAllVars{2}*nAllVars{3}*nAllVars{4} )/prod( nFree );

	if ( length( find( strcmp( 'agent', lower( condFree ) ) ) ) > 1 )

		% If we have two agents against each other
		% we do not want to plot all of free variable 1 against free variable 2,
		% since we will have repetitions. So we do things slightly differently in this case
		% Preallocate space for x- and y-axis points
		sumTotal = sum(1:nFree(1));
		finalX = zeros( sumTotal, arraySizes );
		finalY = zeros( sumTotal, arraySizes );
		legendList = cell( 1, sumTotal );
		hackCounter = 1;
	
		allNames1 = freeNames{1};
		allNames2 = freeNames{2};
		% Loop over all elements in the first free variable
		for agg1Count = 1:nFree(1)
			% Get the X and Y valued point sets
			returnDataXmid = getDataPortion( plot_data.xData, val4dFree(1), agg1Count );
			returnDataYmid = getDataPortion( plot_data.yData, val4dFree(1), agg1Count );
			for agg2Count = agg1Count:nFree(2)
				% Take specific elements of the X and Y valued point sets
				returnDataX = getDataPortion( returnDataXmid, val4dFree(2), agg2Count );
				returnDataY = getDataPortion( returnDataYmid, val4dFree(2), agg2Count );
				% Turn these into vectors
				% We divide by (number iterations)*(whichAgents+1), 
				% since we added number of iterations value and when
				% whichAgents==0, we only added one set of iterations, when
				% whichAgents==1, we added two sets of iterations, corresponding
				% to the row and column play of each agent, hence we divide by 
				% 2 times the number of iterations
				% it means we added only one number to each
				returnDataX = ( returnDataX(:)' )/( nAllVars{5}*(whichAgents+1) );
				returnDataY = ( returnDataY(:)' )/( nAllVars{5}*(whichAgents+1) );
				% Save these to the final storage for x and y values
				finalX( hackCounter, : ) = returnDataX;
				finalY( hackCounter, : ) = returnDataY;
	
				% Update the list of legends
				legendList(1, hackCounter ) = {[allNames1{agg1Count} ' -- ' allNames2{agg2Count}]};
				hackCounter = hackCounter + 1;
			end % for: agg2Count
		end % for: agg1Count
		stVsl = set( stVsl, 'legends', legendList );
	else
		% Not agents plotted against agents, so want all of free variable 1 against all of free variable 2
		% Preallocate space for the x and y values
		finalX = zeros( prod( nFree ), arraySizes );
		finalY = zeros( prod( nFree ), arraySizes );
		legendList = cell( 1, prod( nFree ) );	

		allNames1 = freeNames{1};
		allNames2 = freeNames{2};
		% Loop over all of the elements of the first free variable
		for agg1Count = 1:nFree(1)
			% Take x and y value sets
			returnDataXmid = getDataPortion( plot_data.xData, val4dFree(1), agg1Count );
			returnDataYmid = getDataPortion( plot_data.yData, val4dFree(1), agg1Count );
			for agg2Count = 1:nFree(2)
				% Take specific elements of the x and y value point sets
				returnDataX = getDataPortion( returnDataXmid, val4dFree(2), agg2Count );
				returnDataY = getDataPortion( returnDataYmid, val4dFree(2), agg2Count );
				% Turn these into vectors
				% We divide by (number iterations)*(whichAgents+1), 
				% since we added number of iterations value and when
				% whichAgents==0, we only added one set of iterations, when
				% whichAgents==1, we added two sets of iterations, corresponding
				% to the row and column play of each agent, hence we divide by 
				% 2 times the number of iterations
				% it means we added only one number to each
				returnDataX = ( returnDataX(:)' )/( nAllVars{5}*(whichAgents+1) );
				returnDataY = ( returnDataY(:)' )/( nAllVars{5}*(whichAgents+1) );
	
				% Save into their correct places in the x & y vectors
				finalX( agg2Count + (agg1Count-1)*nFree(2), : ) = returnDataX;
				finalY( agg2Count + (agg1Count-1)*nFree(2), : ) = returnDataY;
	
				% Update the list of legends
				legendList(1, agg2Count + (agg1Count-1)*nFree(2) ) = {[allNames1{agg1Count} ' -- ' allNames2{agg2Count}]};
			end % for: agg2Count
		end % for: agg1Count
		stVsl = set( stVsl, 'legends', legendList );
		finalL = legendList;
	end % ( length( find( strcmp( 'agents', condFree ) ) ) > 1 )
end % one or two free variables

ScatterX = finalX;
ScatterY = finalY;
legGlobal = finalL;

% Put the final set of x and y valued points into a structure
plot_dataStruct = struct( 'xData', finalX, 'yData', finalY );
