function visualObj = visual( varargin )

% Name: visual.m
% Location: MALT/visualClasses/@visual/visual.m
% Purpose: Constructor for the visual class
%          Creates a visual object
% Input Parameters:  no Parameters - default created
%                    1 Parameter - object to copy
%                    8 Parameters - Name for the visualisation method
%                                   Cell list of labelling for parsing
%                                   Cell list of properties for the graph
%                                   Cell list of legend values for the graph
%                                   1 || 0 depicting whether aggregation is expected for all non-selected 
%                                        free variables or not
%                                   Cell list of aggregation methods allowed
%                                   1 || 0 saying whether error bars are allowed or not
%                                   Cell list of free variables not allowed with this visualisation method
%                                   Cell list of extra features required for this visualisation, actually
%                                        variable for possible "extra info"
% Output Parameters: visual object

switch nargin
case 0 % Create default visual object
	visualObj.name = 'visual'; % name of the visualisation method
	visualObj.labelX = ''; % x label for the graph
	visualObj.labelY = ''; % y label for the graph
	visualObj.title = ''; % Title of the graph
	visualObj.gamename = ''; % Name of the game from which results came
	visualObj.propList = {}; % List of graph/plot properties to be changed - cell
	visualObj.propValue = {}; % New values for the properties to be changed - cell
	visualObj.legends = {}; % Legend for the graph - cell
% Now this is the added information for functionality with the various variables/aggregation methods
	visualObj.aggregationForAll = 1; % Expect all non-selected free variables to be aggregated
	visualObj.aggregationAllowed = {'median', 'mean', 'total'}; % Allowed aggregation methods
	visualObj.errorBarsAllowed = 1; % Visualisation allows error bars
	visualObj.freeVarsDisallowed = {}; % List of free variables disallowed with this visualisation
	visualObj.extraFeatures = {}; % List of extra features required for this visualisation
											% This may be a hack to some degree, as will need to check it in a switch
											% statement in the visualisationControl window
	% possible extra features:
	% '1 free' - only works when a single free variable (mainly for scatter)
	% '2 free' - only works with two free variables	(mainly for scatter3)
	visualObj = class( visualObj, 'visual' );
case 1 % Copy another visual object
	if ( isa( varargin{1}, 'visual' ) )
		visualObj = varargin{1};
	else
		error( 'visual.m: Input argument is not a visual object' );
	end
case 9 % Cell list of labelling details, properties and the legend and additional properties
	visualObj.name = varargin{1};
	allLabels = varargin{2};
	if ( ~isa( allLabels, 'cell' ) )
		error( 'visual.m: Second input argument should be a cell of labels' );
	end
	nLabels = length( allLabels );
	% Create empty labels
	visualObj.labelX = '';
	visualObj.labelY = '';
	visualObj.title = '';
	visualObj.gamename = '';
	% Next two lines are for the gca settings
	visualObj.gcaTitles = {};
	visualObj.gcaLabels = {};
	% Update the labels that exist
	for cLab = 1:2:nLabels
		switch lower( allLabels{cLab} )
		case 'labelx'
			visualObj.labelX = allLabels{cLab + 1};
		case 'labely'
			visualObj.labelY = allLabels{cLab + 1};
		case 'title'
			visualObj.title = allLabels{cLab + 1};
		case 'gamename'
			visualObj.gamename = allLabels{cLab + 1};
		otherwise
			sprintf( 'Unknown label: %s\n', allLabels{cLab} );
		end % switch: allLabels{cLab}
	end % for: cLab

	% Add the gamename seperately to the title
	visualObj.title = {visualObj.gamename visualObj.title};

	allProperties = varargin{3};
	if ( ~isa( allProperties, 'cell' ) )
		error( 'visual.m: Third input argument should be a cell of properties and their new values' );
	end
	nProperties = length( allProperties );
	visualObj.propList = cell(1, nProperties/2);
	visualObj.propValue = cell(1, nProperties/2);
	for cProp = 1:2:nProperties
		visualObj.propList( 1, ceil(cProp/2) ) = allProperties{cProp};
		visualObj.propValue( 1, ceil(cProp/2) ) = allProperties{cProp + 1};
	end

	allLegends = varargin{4};
	if ( ~isa( allLegends, 'cell' ) )
		error( 'visual.m: Fourth input argument should be a cell consisting of legend labels' );
	end
	visualObj.legends = allLegends;

% Now this is the added information for functionality with the various variables/aggregation methods
	visualObj.aggregationForAll = varargin{5}; % Expect all non-selected free variables to be aggregated
	visualObj.aggregationAllowed = varargin{6}; % Allowed aggregation methods
	visualObj.errorBarsAllowed = varargin{7}; % Visualisation allows error bars
	visualObj.freeVarsDisallowed = varargin{8}; % List of free variables disallowed with this visualisation
	visualObj.extraFeatures = varargin{9}; % List of extra features required for this visualisation
											% This may be a hack to some degree, as will need to check it in a switch
											% statement in the visualisationControl window
	
	visualObj = class( visualObj, 'visual' );
otherwise
	error_disp = sprintf( 'Valid parameters:\nNo Parameters - default created\n1 Parameter - object to copy\n2 Parameters - Cell list of labelling for parsing\n\tCell list of properties for the graph\n\tCell list of legend values for the graph' );
	disp( error_disp );
	error( 'visual.m: Incorrect number of input arguments' );
end % switch:nargin
